from numba import cuda
import sys, platform

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions

import warnings
import sys
import re
import platform
import numba
from numba import *

_deprecation_warning = "\
The numbapro package is deprecated in favour of the accelerate package. \
Please update your code to use equivalent functions from accelerate."

warnings.showwarning(_deprecation_warning, ImportWarning, __name__, 1)

# Numba version checking
m = re.match(r"(\d+)\.(\d+)\.(\d+).*", numba.__version__)
NUMBA_VERSION_REQ = (0, 22, 0)
if m is None or tuple(map(int, m.groups())) < NUMBA_VERSION_REQ:
    warnings.showwarning(
        "Numba version too old; expecting %d.%d.%d" % NUMBA_VERSION_REQ,
        ImportWarning, __name__, 1)
del m

import numbapro._cuda    # import time sideeffect
from numba.npyufunc import vectorize, guvectorize
import numbapro.vectorizers


def test():
    try:
        print('NumbaPro Tests')

        def failfast(ok):
            if not ok:
                raise Exception('Test Failed')

        print('vectorizers'.center(80, '-'))
        import numbapro.vectorizers.tests
        failfast(numbapro.vectorizers.tests.test())

        check_cuda()

        cuda_compatible = True
        if sys.platform.startswith('darwin'):
            ver = platform.mac_ver()[0]
            major, minor, micro = ver.split('.', 2)
            if (int(major), int(minor)) < (10, 9):
                cuda_compatible = False

        if cuda.is_available() and cuda_compatible:
            print('cudalib'.center(80, '-'))
            import numbapro.cudalib.tests
            failfast(numbapro.cudalib.tests.test())

            print('cudapy'.center(80, '-'))
            import numbapro.cudapy.tests
            failfast(numbapro.cudapy.tests.test())

            print('cudavec'.center(80, '-'))
            import numbapro.cudavec.tests
            failfast(numbapro.cudavec.tests.test())

        else:
            print('Skipped CUDA tests')
    except Exception as e:
        import traceback

        traceback.print_exc()
        print('Test failed')
        return False
    else:
        print('All tests passed')
        return True


def check_cuda():
    from numba.cuda import is_available, cuda_error

    if not is_available():
        print("CUDA is not available")
        print(cuda_error())
        return

    from numba.cuda.cudadrv import libs
    from numba import cuda

    ok = True

    print('libraries detection'.center(80, '-'))
    if not libs.test():
        ok = False

    print('hardware detection'.center(80, '-'))
    if not cuda.detect():
        ok = False

    # result
    if not ok:
        print('FAILED')
    else:
        print('PASSED')
    return ok


def _initialize_all():
    """
    Initialize extensions to Numba.
    For backward compatibility we add support for the 'gpu' target here.
    """
    from numba.npyufunc import Vectorize, GUVectorize

    Vectorize.target_registry.ondemand['gpu'] = Vectorize.target_registry.ondemand['cuda']
    GUVectorize.target_registry.ondemand['gpu'] = GUVectorize.target_registry.ondemand['cuda']

_initialize_all()
del _initialize_all

